module LatexPropositions 
  (module PropositionalLogic, module LatexPropositions) where

import PropositionalLogic
import System.Cmd
import System.Posix.Process

latexStart 
 = "\\documentclass[10pt,a4paper,oneside]{article}\n\
   \\\usepackage[german]{babel}\n\
   \\\setlength{\\parindent}{0pt}\n\
   \\\setlength{\\parskip}{1ex plus 0.5ex minus 0.2ex}\n\
   \\\usepackage{qtree}\n\
   \\\usepackage{amssymb}\n\
   \\\begin{document}\n"

latexEnd = "\n\\end{document}"

pdfLatex file = rawSystem "pdflatex" [file]

showPDF file  = do
  ex <- rawSystem "acroread" [file]
  forkProcess $return ()

latex  = latexFile "test"

latexFile fileName title content = do
  writeFile (fileName++".tex") 
    (latexStart++"\\title{"++title
               ++"}\n\\maketitle\n\n"++content++latexEnd)
  pdfLatex (fileName++".tex") 
  showPDF (fileName++".pdf") 


latexAll = latex "alle Formeln" $toLatex$take 500$allformulae ["A","B","C"]

m3 taut =  content
 where 
  f1 = Not taut
  f2 = noArrows f1
  f3 = notInside f2
  f4 = clauselForm f3
  f5 = mkClauses f4
  content = 
        "Als Tautologie zu beweisende Formel: \\\\\n$"
     ++ toLatex taut++"$\\\\\n"
     ++ "Negat der Formel:\\\\\n$"++toLatex f1++"$\\\\\n"
     ++ "Elemination der Pfeiloperatoren:\\\\\n$"++toLatex f2++"$\\\\\n"
     ++ "Negation direkt vor die Atome:\\\\\n$"++toLatex f3++"$\\\\\n"
     ++ "Klauselform:\\\\\n$"++toLatex f4++"$\\\\\n"
     ++ "Klauselmenge:\\\\\n$"++toLatex f5++"$\\\\\n"
     ++ "Beweis:\\\\\n"++(showProof$snd$head$proofs f5)
     
m4 ts= latex "Resolutionsbeweise von Tautologien der Aussagenlogik"
       $foldr1 (\x y -> x++"\\eject\n"++y) (map m3 ts)

m5 = m4 ts

natweis fileName f
  = latexFile 
      fileName
      ("Ableitungskette f\\\"ur: $"++toLatex f++"$") -- "
      (concat$take 100000 ["$"++toLatex f1++"$\n\n"|f1<-beweis f])

