{-# OPTIONS -fglasgow-exts #-}

module PropositionalLogic where
import Maybe
import List
import Data.Generics

data Prop
 =  Atom String
   |Not Prop
   |Prop :-> Prop

     -- erweiterte abkürzende Syntax um Konjunktion, 
     -- Disjunktion und Äquivalenz
   |Prop :/\ Prop
   |Prop :\/ Prop 
   |Prop :<-> Prop

     deriving (Eq,Show,Typeable,Data)

infixl 8 :/\
infixl 6 :\/
infixl 4 :->
infixl 2 :<->

isAtom (Atom _) = True
isAtom _ = False

a = Atom "A"
b = Atom "B"
c = Atom "C"
d = Atom "D"
e = Atom "E"

ex1 = a
ex2 = Not a
ex3 = ex1 :\/ ex2
ex4 = ex1 :-> ex2
ex5 = ex3 :-> b
ex6 = Not b :\/ ex5 
ex7 = a :<-> b
ex8 = ex7 :<-> ex7
ex9 = ex7 :/\ Not ex7
ex10 = a :/\ (b:->c)
ex11 = a :/\ (Not a)
ex12 = ex8 :<-> ex8

exs=[ex1,ex2,ex3,ex4,ex5,ex6,ex7,ex8,ex9,ex10,ex11,ex12]

class ToLatex a where
    toLatex:: a -> String

instance ToLatex Prop where
    toLatex (Atom x) = "\\mbox{\\sl "++x++"}"
    toLatex (Not p) = "\\neg "++toLatex p
    toLatex (p1 :-> p2) 
     = "("++toLatex p1++" \\rightarrow "++toLatex p2++")"
    toLatex (p1 :<-> p2)
      = "("++toLatex p1++" \\leftrightarrow "++toLatex p2++")"
    toLatex (p1 :/\ p2)
      = "("++toLatex p1++" \\wedge "++toLatex p2++")"
    toLatex (p1 :\/ p2)
      = "("++toLatex p1++" \\vee "++toLatex p2++")"

instance ToLatex [Prop] where
  toLatex ps = concat ["$"++toLatex p++"$\n\n"|p<-ps]


simplify :: Prop -> Prop
simplify = everywhere$mkT s
 where
   s (p1 :\/ p2)  = Not p1 :-> p2
   s (p1 :/\ p2)  = Not (s (Not p1 :\/ Not p2))
   s (p1 :<-> p2) = simplify ((Not p1:\/p2) :/\ (Not p2:\/p1))
   s x = x

getAtoms :: Prop -> [String]
getAtoms= nub . everything (++) ([] `mkQ` atom)               
  where
    atom (Atom x) = [x]
    atom _ = []

data Wahrheit = W|F deriving (Eq, Enum,Show)

type Interpretation = [(String,Wahrheit)]

interpret it (Atom x) = fromJust$lookup x it

interpret it (Not p) 
 |interpret it p == W = F
 |otherwise = W

interpret it (p1 :-> p2)
 |interpret it p1 == F = W
 |otherwise = interpret it p2

interpret it p = interpret it (simplify p)

allInterpretations p = [interpret int p==W|int <- inter$getAtoms p]
  where
   inter []     = [[]]
   inter (a:as) = concat [[(a,W):ai,(a,F):ai]|ai <- inter as]

tautology  = and . allInterpretations

erfuellbar  = or . allInterpretations

unerfuellbar = not . erfuellbar

t01 = a :/\ a :<-> a
t02 = a :\/ a :<-> a
t03 = (a :/\ b) :/\ c :<-> a :/\ (b :/\ c)
t04 = (a :\/ b) :\/ c :<-> a :\/ (b :\/ c)
t05 = a :/\ b :<-> b:/\a
t06 = a :\/ b :<-> b:\/a
t07 = a :/\ (b:\/c) :<-> (a:/\b):\/(a:/\c)
t08 = a :\/ (b:/\c) :<-> (a:\/b):/\(a:\/c)
t09 = a :/\ (a:\/b) :<-> a
t10 = a :\/ (a:/\b) :<-> a
t11 = Not(Not a) :<-> a
t12 = Not (a:/\b) :<-> Not a:\/ Not b
t13 = Not (a:\/b) :<-> Not a:/\ Not b

ts=[t01,t02,t03,t05,t06,t07,t08,t09,t10,t11,t12,t13]

atom = map Atom 
negated = map Not
implies xs ys = [a:->b|a<-xs,b<-ys,not (a==b)] 

newgen sofar lastnew
  = lastnew
     ++newgen 
      (sofar++lastnew) 
      (    negated lastnew
        ++ implies lastnew lastnew 
        ++ [a:->a|a<-lastnew]
        ++ implies sofar lastnew
        ++ implies lastnew sofar )

allformulae vars = newgen [] (atom vars)

diagonal (x:xs) (y:ys) = (x,y):merge3 xs2 ys2 dia2
 where
  dia2= diagonal xs ys 
  xs2 = [(x,y)|y<-ys]
  ys2 = [(x,y)|x<-xs] 

diagonal3 xs ys zs = diagonal xs$diagonal ys zs

merge (x:xs) (y:ys)= x:y:merge xs ys
merge xs ys = xs++ys

merge3 xs ys zs = merge xs$merge ys zs

ax1 a b = a:-> (b:->a)
ax2 a b c = (a:->(b:->c)):->((a:->b):->(a:->c))
ax3 a b = (Not a:-> Not b):->(b:->a)

axioms1 fors = map (\(a,b)->ax1 a b)$diagonal fors fors
axioms3 fors = map (\(a,b)->ax3 a b)$diagonal fors fors
axioms2 fors = map (\(a,(b,c))->ax2 a b c)$diagonal3 fors fors fors

allAxioms sigs = merge3 (axioms1 fs) (axioms2 fs) (axioms3 fs)
 where
   fs= allformulae sigs

mp a1 (a2:->b)
 |a1==a2=[b]
mp (a2:->b) a1
 |a1==a2=[b]
mp _ _ =[]

mps x = concat . (map (mp x))

allTauts vars = newgen [] axs
  where
   axs = allAxioms vars
   newgen old (n:ext) = n:(newgen ((n:old)) (ponens++ext))
     where
       ponens = [x|x<-mps n old,not (elem x old )]

allts = allTauts ["A","B","C"]
beweis f = (takeWhile (\x->not (x==f)) (allts))++[f] 

f00 = a:->a 
f01 = (a:->b):->((b:->c):->(a:->c)) 
f02 = a:->((a:->b):->b)
f03 = Not(Not a) :-> a
f04 = a:-> Not(Not a)
f05 = (a:->b):->(Not b:-> Not a)
f06 = a:-> (Not b :-> Not (a:->b))
f07 = Not a :-> (a:->b)
f08 = a :-> (b:-> (a:->b))
f09 = Not (a:-> b) :-> a
f10 = Not (a:-> b) :-> Not b
f11 = (a:->Not a):-> Not a
f12 = (Not a:->a):-> a
f13 = (a:->b):->((Not a:-> b):->b)
f14 = Not (a:->a):->b

type Clause a = ([a],[a])
type Clauses a = [Clause a]

instance ToLatex (Clause String) where
  toLatex ([],[])   = "\\Box"
  toLatex (pos,neg) = "\\{"++ concatWith ", " (pos++map ((++)"\\neg ") neg)
                           ++ "\\}"

instance ToLatex (Clauses String) where
  toLatex cs = "\\{"++ concatWith ", " (map toLatex  cs)++"\\}"

concatWith sep [] = ""
concatWith sep [a] = a
concatWith sep (x:xs) = x++sep++concatWith sep xs

(+++) (a1,a2) (b1,b2) = (nub (a1++b1),nub (a2++b2))

noArrows :: Prop -> Prop
noArrows = everywhere (mkT noA) where
  noA (a :-> b) = Not a :\/ b
  noA (a :<-> b) = (Not a :\/ b) :/\ (Not b :\/ a)
  noA x = x

notInside :: Prop -> Prop
notInside = (everywhere (mkT noI)) where
   noI (Not (Not p))    = p
   noI (Not (p1:\/p2))  = (noI (Not p1):/\ noI(Not p2))
   noI (Not (p1:/\p2))  = (noI (Not p1):\/ noI(Not p2))
   noI x = x

clauselForm = everywhere$mkT cf where
     cf (a:\/(b:/\c))    = clauselForm ((a:\/b) :/\ (a:\/c))
     cf ((b:/\c):\/a)    = cf (a:\/(b:/\c))
     cf x                = x

clauses = cf  where
     cf (Atom x)       = [([x],[])]
     cf (Not (Atom x)) = [([],[x])]
     cf (a:/\b)          = clauses a++clauses b
     cf (a:\/b)          = [ca+++cb|ca<-clauses a,cb<-clauses b]

inCls=clauselForm.notInside.noArrows
mkClauses = clauses.inCls

resolve c1@(p1,n1) c2@(p2,n2)
 |c1==c2 =[]
 |otherwise = [(nub((p1\\[l])++p2),nub(n1++(n2\\[l])))|l<-p1,elem l n2]

breadth f xs =  [f x y|x<-xs,y<-xs]

tautologyClause (pos,neg) = or [at1==at2|at1<-pos,at2<-neg]

allresolvents xs = xs++(concat$ breadth resolve xs)

proof p = elem ([],[])(concat(iterate allresolvents 
                       (filter (not.tautologyClause)$mkClauses p)))

data T a = TN|TC (T a) a (T a) deriving (Eq, Show,Data,Typeable)

resolveT (c1@(p1,n1),t1) (c2@(p2,n2),t2)
 = [(x,TC t1 x t2)|l<-p1
                  ,elem l n2
                  ,x<-[(nub((p1\\[l])++p2),nub(n1++(n2\\[l])))]]

allresolventsT xs = xs++(concat$ breadth resolveT xs)

proofs p = filter (\(x,_) -> x==([],[]))
     (concat(iterate allresolventsT (map (\x->(x,TC TN x TN)) p)))

showProof TN = ""
showProof (TC TN x TN) = "$"++toLatex x++"$"
showProof (TC l x r)
  =   showProof l ++"\\\\\n"
    ++showProof r ++"\\\\\n"
    ++"$"++toLatex x++"$" 

moin e = writeFile "test"$showProof$snd$head$proofs$mkClauses$Not e

data Tableau = Node Prop [Tableau] [Prop]
     deriving (Eq,Show,Typeable,Data)

closed (Node f [] ancestors) = contradiction (f:ancestors)

closed (Node f children _) = and(map closed children)

contradiction xs = or[elem (Not x) xs|x<-xs,isAtom x]

addChildren xs (Node f children ancestors) 
 = Node f ([(Node x [] (f:ancestors))|x<-xs]++children) ancestors

addToLeaves xs t@(Node f [] ancestors) 
 |closed t  = t
 |otherwise =  addChildren xs t

addToLeaves xs (Node f children ancestors) 
  = Node f (map (addToLeaves xs) children) ancestors

expand t@(Node (Not(Not a)) children ancestors)
 = addToLeaves [a] t
expand t@(Node (a:/\b) children ancestors)
 = addToLeaves [b]$ addToLeaves [a] t
expand t@(Node (Not (a:\/b)) children ancestors)
 = addToLeaves [Not b]$ addToLeaves [Not a] t
expand t@(Node (Not (a:->b)) children ancestors)
 = addToLeaves [a]$ addToLeaves [Not b] t

expand t@(Node (a:\/b) children ancestors)
 = addToLeaves [a,b] t
expand t@(Node (Not (a:/\b)) children ancestors)
 = addToLeaves [Not a,Not b] t
expand t@(Node (a:->b) children ancestors)
 = addToLeaves [Not a,b] t

expand t = t

expandAll = expandChildren.expand

expandChildren (Node f children ancestors)
 = Node f (map expandAll children) ancestors

tableau f = expandAll (Node f [] [])

tableauProof = tableau.Not

isTableauProven = closed.tableauProof

instance ToLatex Tableau where
    toLatex t
      = "\\Tree "++aux t++""
      where 
        aux (Node f [] ancestors) 
         |contradiction (f:ancestors) 
           = "[ .\\mbox{$"++toLatex f++"$}\n \\mbox{$\\Box$}\n] "
         |otherwise = " \\mbox{$"++toLatex f++"$} "
        aux (Node f cs ancestors)
          = "[ .\\mbox{$"++toLatex f++"$}\n "++concat(map aux cs)++" ]"


